@@ -30,12 +30,13 @@ import (
 	"energi.world/core/gen3/core"
 	"energi.world/core/gen3/core/state"
 	"energi.world/core/gen3/core/types"
+	energi_consensus "energi.world/core/gen3/energi/consensus"
 	"energi.world/core/gen3/event"
 	"energi.world/core/gen3/log"
 	"energi.world/core/gen3/params"
 	mapset "github.com/deckarep/golang-set"
 
-	energi_consensus "energi.world/core/gen3/energi/consensus"
+	energi_params "energi.world/core/gen3/energi/params"
 )
 
 const (
@@ -175,6 +176,10 @@ type worker struct {
 	running int32 // The indicator whether the consensus engine is running or not.
 	newTxs  int32 // New arrival transaction count since last sealing work submitting.
 
+	// Metrics calculation
+	lastBlockTime   uint64
+	lastBlockNumber uint64
+
 	// External functions
 	isLocalBlock func(block *types.Block) bool // Function used to determine whether the specified block is mined by local miner.
 
@@ -230,7 +235,7 @@ func newWorker(config *params.ChainConfig, engine consensus.Engine, eth Backend,
 		Number:     num.Add(num, common.Big1),
 		GasLimit:   core.CalcGasLimit(parent, worker.gasFloor, worker.gasCeil),
 		Extra:      worker.extra,
-		Time:       uint64(parent.Time() + energi_consensus.MinBlockGap),
+		Time:       uint64(parent.Time() + energi_params.MinBlockGap),
 	}
 	if err := worker.makeCurrent(parent, header); err != nil {
 		panic(err)
@@ -889,7 +894,7 @@ func (w *worker) commitNewWork(interrupt *int32, noempty bool, timestamp int64)
 		Number:     num.Add(num, common.Big1),
 		GasLimit:   core.CalcGasLimit(parent, w.gasFloor, w.gasCeil),
 		Extra:      w.extra,
-		Time:       uint64(parent.Time() + energi_consensus.MinBlockGap),
+		Time:       uint64(parent.Time() + energi_params.MinBlockGap),
 	}
 	// Only set the coinbase if our consensus engine is running (avoid spurious block rewards)
 	if w.isRunning() {
@@ -1076,6 +1081,20 @@ func (w *worker) commit(uncles []*types.Header, interval func(), update bool, st
 			log.Info("Commit new mining work", "number", block.Number(), "sealhash", w.engine.SealHash(block.Header()),
 				"uncles", len(uncles), "txs", w.current.tcount, "gas", block.GasUsed(), "fees", feesEth, "elapsed", common.PrettyDuration(time.Since(start)))
 
+			if w.lastBlockNumber == 0 {
+				// initialize from first processed block
+				w.lastBlockTime = block.Time()
+				w.lastBlockNumber = block.Number().Uint64()
+			} else if w.lastBlockNumber != block.Number().Uint64() {
+				blocktimeGauge.Update(int64(block.Time() - w.lastBlockTime))
+				blocksizeGauge.Update(int64(block.Size()))
+				difficultyGauge.Update(block.Difficulty().Int64())
+				w.lastBlockTime = block.Time()
+				w.lastBlockNumber = block.Number().Uint64()
+			} else {
+				transactionGauge.Update(int64(block.Transactions().Len()))
+			}
+
 		case <-w.exitCh:
 			log.Info("Worker has exited")
 		}
