@@ -27,7 +27,7 @@ import (
 // Genesis hashes to enforce below configs on.
 var (
 	MainnetGenesisHash = common.HexToHash("0xd8e2a3b0ad08f8eaabaf653d25b7d9beee2911101010a2cd4f6692a9a1dc228a")
-	TestnetGenesisHash = common.HexToHash("0x4d5513e2cd959af850591de3e38f6cc7a2f9775f8159da0159788a95ef4e85ac")
+	TestnetGenesisHash = common.HexToHash("0x93b3a19ffad91326cd638f15930fdea1268c24d599b50b0e19299209c70c0e4e")
 
 	MainnetMigrationSigner = common.HexToAddress("0xac34a2555de08384cd7960f35d3ab048fcf9f83a")
 	TestnetMigrationSigner = common.HexToAddress("0xb1372ea07f6a92bc86fd5f8cdf468528f79f87ca")
@@ -43,6 +43,9 @@ var (
 
 	MainnetBackbone = common.HexToAddress("0x79C7CF016E53e5C47906c2daF6De2aA00AAcdB1e")
 	TestnetBackbone = common.HexToAddress("0x5143c57fcde025f05a19d0de9a7dac852e553624")
+
+	MainnetHFProxy = common.HexToAddress("0xe2616f793916A0BD9C66939c08c94693d483df03")
+	TestnetHFProxy = common.HexToAddress("0x886c71F1Af530478204dD12fB0BA34A46899C16D")
 )
 
 // TrustedCheckpoints associates each known checkpoint with the genesis hash of
@@ -76,11 +79,12 @@ var (
 		ConstantinopleBlock: big.NewInt(0),
 		PetersburgBlock:     big.NewInt(0),
 		Energi: &EnergiConfig{
-			BackboneAddress: MainnetBackbone,
-			MigrationSigner: MainnetMigrationSigner,
-			EBISigner:       MainnetEBISigner,
-			CPPSigner:       MainnetCPPSigner,
-			HFSigner:        MainnetHFSigner,
+			BackboneAddress:              MainnetBackbone,
+			MigrationSigner:              MainnetMigrationSigner,
+			EBISigner:                    MainnetEBISigner,
+			CPPSigner:                    MainnetCPPSigner,
+			HFSigner:                     MainnetHFSigner,
+			HardforkRegistryProxyAddress: MainnetHFProxy,
 		},
 		SuperblockCycle:     big.NewInt(60 * 24 * 14), // A super block happens at the end of every 20160 block (Approx. 14 days)
 		MNRequireValidation: big.NewInt(10),
@@ -88,8 +92,6 @@ var (
 		MNCleanupPeriod:     big.NewInt(60 * 60 * 24 * 14), // Inactive MN denounced after 1209600 sec (14 days/ 2 weeks)
 		MNEverCollateral:    new(big.Int).Mul(big.NewInt(3000000), big.NewInt(Ether)),
 		MNRewardsPerBlock:   big.NewInt(10), // MN with the minimum collateral amount gets a block reward of (9.14/10) 0.914 NRG.
-
-		HardforkRegistryProxyAddress: common.Address{},
 		HFFinalizationPeriod: big.NewInt(30), // The hardfork should be finalized in 30 blocks.
 	}
 
@@ -104,11 +106,12 @@ var (
 		ConstantinopleBlock: big.NewInt(0),
 		PetersburgBlock:     big.NewInt(0),
 		Energi: &EnergiConfig{
-			BackboneAddress: TestnetBackbone,
-			MigrationSigner: TestnetMigrationSigner,
-			EBISigner:       TestnetEBISigner,
-			CPPSigner:       TestnetCPPSigner,
-			HFSigner:        TestnetHFSigner,
+			BackboneAddress:              TestnetBackbone,
+			MigrationSigner:              TestnetMigrationSigner,
+			EBISigner:                    TestnetEBISigner,
+			CPPSigner:                    TestnetCPPSigner,
+			HFSigner:                     TestnetHFSigner,
+			HardforkRegistryProxyAddress: TestnetHFProxy,
 		},
 		SuperblockCycle:     big.NewInt(60 * 24),
 		MNRequireValidation: big.NewInt(5),
@@ -116,8 +119,6 @@ var (
 		MNCleanupPeriod:     big.NewInt(60 * 60 * 3),
 		MNEverCollateral:    new(big.Int).Mul(big.NewInt(30000), big.NewInt(Ether)),
 		MNRewardsPerBlock:   big.NewInt(10),
-
-		HardforkRegistryProxyAddress: common.Address{},
 		HFFinalizationPeriod: big.NewInt(10), // The hardfork should be finalized in 10 blocks.
 	}
 
@@ -133,6 +134,9 @@ var (
 		ConstantinopleBlock: big.NewInt(0),
 		PetersburgBlock:     big.NewInt(0),
 		Ethash:              new(EthashConfig),
+		Energi: &EnergiConfig{
+			HardforkRegistryProxyAddress: common.Address{},
+		},
 	}
 
 	// MainnetTrustedCheckpoint contains the light client trusted checkpoint for the main network.
@@ -156,6 +160,9 @@ var (
 		ConstantinopleBlock: big.NewInt(0),
 		PetersburgBlock:     big.NewInt(0),
 		Ethash:              new(EthashConfig),
+		Energi: &EnergiConfig{
+			HardforkRegistryProxyAddress: common.Address{},
+		},
 	}
 
 	// TestnetTrustedCheckpoint contains the light client trusted checkpoint for the Ropsten test network.
@@ -172,16 +179,16 @@ var (
 	//
 	// This configuration is intentionally not using keyed fields to force anyone
 	// adding flags to the config to also have to set these fields.
-	AllEthashProtocolChanges = &ChainConfig{big.NewInt(1337), big.NewInt(0), big.NewInt(0), common.Hash{}, big.NewInt(0), big.NewInt(0), big.NewInt(0), big.NewInt(0), big.NewInt(0), nil, new(EthashConfig), nil, nil, big.NewInt(0), big.NewInt(0), big.NewInt(0), big.NewInt(0), big.NewInt(0), big.NewInt(0), common.Address{}, big.NewInt(0)}
+	AllEthashProtocolChanges = &ChainConfig{big.NewInt(1337), big.NewInt(0), big.NewInt(0), common.Hash{}, big.NewInt(0), big.NewInt(0), big.NewInt(0), big.NewInt(0), big.NewInt(0), nil, new(EthashConfig), nil, &EnergiConfig{ HardforkRegistryProxyAddress: common.Address{} }, big.NewInt(0), big.NewInt(0), big.NewInt(0), big.NewInt(0), big.NewInt(0), big.NewInt(0), big.NewInt(0)}
 
 	// AllCliqueProtocolChanges contains every protocol change (EIPs) introduced
 	// and accepted by the Ethereum core developers into the Clique consensus.
 	//
 	// This configuration is intentionally not using keyed fields to force anyone
 	// adding flags to the config to also have to set these fields.
-	AllCliqueProtocolChanges = &ChainConfig{big.NewInt(1337), big.NewInt(0), big.NewInt(0), common.Hash{}, big.NewInt(0), big.NewInt(0), big.NewInt(0), big.NewInt(0), big.NewInt(0), nil, nil, &CliqueConfig{Period: 0, Epoch: 30000}, nil, big.NewInt(0), big.NewInt(0), big.NewInt(0), big.NewInt(0), big.NewInt(0), big.NewInt(0), common.Address{}, big.NewInt(0)}
+	AllCliqueProtocolChanges = &ChainConfig{big.NewInt(1337), big.NewInt(0), big.NewInt(0), common.Hash{}, big.NewInt(0), big.NewInt(0), big.NewInt(0), big.NewInt(0), big.NewInt(0), nil, nil, &CliqueConfig{Period: 0, Epoch: 30000}, &EnergiConfig{ HardforkRegistryProxyAddress: common.Address{} }, big.NewInt(0), big.NewInt(0), big.NewInt(0), big.NewInt(0), big.NewInt(0), big.NewInt(0), big.NewInt(0)}
 
-	TestChainConfig = &ChainConfig{big.NewInt(1), big.NewInt(0), big.NewInt(0), common.Hash{}, big.NewInt(0), big.NewInt(0), big.NewInt(0), big.NewInt(0), big.NewInt(0), nil, new(EthashConfig), nil, nil, big.NewInt(0), big.NewInt(0), big.NewInt(0), big.NewInt(0), big.NewInt(0), big.NewInt(0), common.Address{}, big.NewInt(0)}
+	TestChainConfig = &ChainConfig{big.NewInt(1), big.NewInt(0), big.NewInt(0), common.Hash{}, big.NewInt(0), big.NewInt(0), big.NewInt(0), big.NewInt(0), big.NewInt(0), nil, new(EthashConfig), nil, &EnergiConfig{ HardforkRegistryProxyAddress: common.Address{} }, big.NewInt(0), big.NewInt(0), big.NewInt(0), big.NewInt(0), big.NewInt(0), big.NewInt(0), big.NewInt(0)}
 	TestRules       = TestChainConfig.Rules(new(big.Int))
 )
 
@@ -245,10 +252,6 @@ type ChainConfig struct {
 	// MNRewardsPerBlock defines the fraction of the total MN reward per block share
 	// payable to the MN holding the minimum amount of collateral.
 	MNRewardsPerBlock *big.Int `json:"mnRewardsPerBlock"`
-	// HardforkRegistryProxyAddress is the address of the proxy contract for
-	// the HardforkRegistry. This contract was not deployed via genesis so it can change
-	// between networks.
-	HardforkRegistryProxyAddress common.Address `json:"hfRegistryProxyAddress"`
 	// HFFinalizationPeriod is the number of blocks after the hardfork block,
 	// within which a given hardfork must be finalized and made immutable or
 	// rendered invalid and editable.
@@ -276,11 +279,12 @@ func (c *CliqueConfig) String() string {
 
 // EnergiConfig is the consensus engine config for proof-of-stake based sealing.
 type EnergiConfig struct {
-	BackboneAddress common.Address `json:"backboneAddress"`
-	MigrationSigner common.Address `json:"migrationSigner"`
-	EBISigner       common.Address `json:"ebiSigner"`
-	CPPSigner       common.Address `json:"cppSigner"`
-	HFSigner        common.Address `json:"hfSigner"`
+	BackboneAddress               common.Address `json:"backboneAddress"`
+	MigrationSigner               common.Address `json:"migrationSigner"`
+	EBISigner                     common.Address `json:"ebiSigner"`
+	CPPSigner                     common.Address `json:"cppSigner"`
+	HFSigner                      common.Address `json:"hfSigner"`
+	HardforkRegistryProxyAddress  common.Address `json:"hfProxyAddress"`
 }
 
 // String implements the stringer interface, returning the consensus engine details.
