@@ -18,16 +18,12 @@
 package tests
 
 import (
-	"os"
 	"testing"
 
 	"energi.world/core/gen3/core/vm"
 )
 
 func TestVM(t *testing.T) {
-	if val, ok := os.LookupEnv("SKIP_KNOWN_FAIL"); ok && val == "1" {
-		t.Skip("unit test is broken: conditional test skipping activated")
-	}
 	t.Parallel()
 	vmt := new(testMatcher)
 	vmt.slow("^vmPerformance")
